@@ -1,6 +1,8 @@
-#!/usr/bin/env python
+# -*- Mode: Python -*-
+# vi:si:et:sw=4:sts=4:ts=4
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2004, MetaSlash Inc.  All rights reserved.
+# Portions Copyright (c) 2005, Google, Inc. All rights reserved.
 
 """
 Setup a lot of info about Python builtin types, functions, methods, etc.
@@ -78,7 +80,7 @@ if utils.pythonVersion() >= utils.PYTHON_2_2 :
     GLOBAL_FUNC_INFO['compile'] = (types.CodeType, 3, 5)
     GLOBAL_FUNC_INFO['dict'] = (types.DictType, 0, 1, ['items'])
     GLOBAL_FUNC_INFO['file'] = GLOBAL_FUNC_INFO['open']
-    GLOBAL_FUNC_INFO['float'] = (types.IntType, 0, 1, ['x'])
+    GLOBAL_FUNC_INFO['float'] = (types.FloatType, 0, 1, ['x'])
     GLOBAL_FUNC_INFO['int'] = (types.IntType, 0, 2, ['x'])
     GLOBAL_FUNC_INFO['list'] = (types.ListType, 0, 1, ['sequence'])
     GLOBAL_FUNC_INFO['long'] = (types.LongType, 0, 2, ['x'])
@@ -96,11 +98,32 @@ if utils.pythonVersion() >= utils.PYTHON_2_2 :
 
     GLOBAL_FUNC_INFO['bool'] = (BOOL, 1, 1, ['x'])
 
-if globals().has_key('zip') :
-    GLOBAL_FUNC_INFO['zip'] = (types.ListType, 1, None)
+if utils.pythonVersion() >= utils.PYTHON_2_3:
+    GLOBAL_FUNC_INFO['dict'] = (types.DictType, 0, 1, [])
 
-if globals().has_key('enumerate'):
-    GLOBAL_FUNC_INFO['enumerate'] = (types.TupleType, 1, 1, ['sequence'])
+if utils.pythonVersion() >= utils.PYTHON_2_5:
+    GLOBAL_FUNC_INFO['max'] = (Stack.TYPE_UNKNOWN, 1, None, ['key'])
+    GLOBAL_FUNC_INFO['min'] = (Stack.TYPE_UNKNOWN, 1, None, ['key'])
+
+def tryAddGlobal(name, *args):
+    if globals().has_key(name):
+        GLOBAL_FUNC_INFO[name] = args
+
+zipMinArgs = 1
+if utils.pythonVersion() >= utils.PYTHON_2_4:
+    zipMinArgs = 0
+    
+tryAddGlobal('zip', types.ListType, zipMinArgs, None)
+
+tryAddGlobal('enumerate', types.TupleType, 1, 1, ['sequence'])
+# sum() could also return float/long
+tryAddGlobal('sum', types.IntType, 1, 2, ['start'])
+# sorted() and reversed() always return an iterator  (FIXME: support iterator)
+tryAddGlobal('sorted', Stack.TYPE_UNKNOWN, 1, 1)
+tryAddGlobal('reversed', Stack.TYPE_UNKNOWN, 1, 1)
+
+tryAddGlobal('all', BOOL, 1, 1)
+tryAddGlobal('any', BOOL, 1, 1)
 
 _STRING_METHODS = { 'capitalize': (types.StringType, 0, 0),
                     'center': (types.StringType, 1, 1),
@@ -138,6 +161,13 @@ _STRING_METHODS = { 'capitalize': (types.StringType, 0, 0),
 
 if utils.pythonVersion() >= utils.PYTHON_2_2 :
     _STRING_METHODS['decode'] = (types.UnicodeType, 0, 2)
+    _STRING_METHODS['zfill'] = (types.StringType, 1, 1)
+
+if utils.pythonVersion() >= utils.PYTHON_2_4:
+    _STRING_METHODS['rsplit'] = (types.StringType, 0, 2)
+    _STRING_METHODS['center'] = (types.StringType, 1, 2),
+    _STRING_METHODS['ljust'] = (types.StringType, 1, 2),
+    _STRING_METHODS['rjust'] = (types.StringType, 1, 2),
 
 BUILTIN_METHODS = { types.DictType :
                     { 'clear': (types.NoneType, 0, 0),
@@ -180,8 +210,29 @@ BUILTIN_METHODS = { types.DictType :
                     },
                   }
 
+if utils.pythonVersion() >= utils.PYTHON_2_4:
+    GLOBAL_FUNC_INFO['set'] = (Stack.TYPE_UNKNOWN, 0, 1)
+    GLOBAL_FUNC_INFO['frozenset'] = (Stack.TYPE_UNKNOWN, 0, 1)
+
+    kwargs = ['cmp', 'key', 'reverse']
+    BUILTIN_METHODS[types.ListType]['sort'] = (types.NoneType, 0, 3, kwargs)
+    BUILTIN_METHODS[types.DictType]['update'] = (types.NoneType, 1, 1, [])
+
 if hasattr({}, 'pop'):
-    BUILTIN_METHODS[types.DictType]['pop'] = (Stack.TYPE_UNKNOWN, 1, 1)
+    BUILTIN_METHODS[types.DictType]['pop'] = (Stack.TYPE_UNKNOWN, 1, 2)
+
+if utils.pythonVersion() >= utils.PYTHON_2_5:
+    _STRING_METHODS['partition'] = (types.TupleType, 1, 1)
+    _STRING_METHODS['rpartition'] = (types.TupleType, 1, 1)
+
+if utils.pythonVersion() >= utils.PYTHON_2_6:
+    GLOBAL_FUNC_INFO['bin'] = (types.StringType, 1, 1)
+    GLOBAL_FUNC_INFO['bytesarray'] = (bytearray, 0, 1)
+    GLOBAL_FUNC_INFO['bytes'] = (bytes, 0, 1)
+    GLOBAL_FUNC_INFO['format'] = (types.StringType, 1, 2)
+    GLOBAL_FUNC_INFO['next'] = (Stack.TYPE_UNKNOWN, 1, 2)
+    GLOBAL_FUNC_INFO['print'] = (types.NoneType, 0, None,
+                                 ['sep', 'end', 'file'])
 
 def _setupBuiltinMethods() :
     if utils.pythonVersion() >= utils.PYTHON_2_2 :
@@ -211,7 +262,7 @@ _setupBuiltinMethods()
 MUTABLE_TYPES = (types.ListType, types.DictType, types.InstanceType,)
 
 # identifiers which will become a keyword in a future version
-FUTURE_KEYWORDS = { 'yield': '2.2' }
+FUTURE_KEYWORDS = { 'yield': '2.2', 'with': '2.5', 'as': '2.5' }
 
 METHODLESS_OBJECTS = { types.NoneType : None, types.IntType : None,
                        types.LongType : None, types.FloatType : None,
@@ -220,9 +271,9 @@ METHODLESS_OBJECTS = { types.NoneType : None, types.IntType : None,
                      }
 
 def _setupBuiltinAttrs() :
-    w = Warning.Warning('', 0, '')
-    BUILTIN_ATTRS[types.MethodType] = dir(w.__init__)
-    del w
+    item = Stack.Item(None, None)
+    BUILTIN_ATTRS[types.MethodType] = dir(item.__init__)
+    del item
 
     if utils.pythonVersion() >= utils.PYTHON_2_2 :
         # FIXME: I'm sure more types need to be added here
@@ -280,7 +331,7 @@ _setupBuiltinAttrs()
 
 PENDING_DEPRECATED_MODULES = { 'string': None, 'types': None,
                              }
-DEPRECATED_MODULES = { 'audioop': None, 'FCNTL': 'fcntl', 'gopherlib': None,
+DEPRECATED_MODULES = { 'FCNTL': 'fcntl', 'gopherlib': None,
                        'macfs': 'Carbon.File or Carbon.Folder',
                        'posixfile': 'fcntl', 'pre': None, 'regsub': 're',
                        'statcache': 'os.stat()',
@@ -329,7 +380,6 @@ SPECIAL_METHODS = {
     '__len__': 1,
     '__new__': None,			# new-style class constructor
     '__nonzero__': 1,
-    '__reduce__': 1,
 
     '__hex__': 1,
     '__oct__': 1,
@@ -338,12 +388,14 @@ SPECIAL_METHODS = {
 
     '__invert__': 1,	'__neg__': 1,	'__pos__': 1,     '__abs__': 1,     
     '__complex__': 1,	'__int__': 1,	'__long__': 1,    '__float__': 1,
+    '__unicode__': 1,
 
     '__eq__': 2,	'__ne__': 2,
     '__ge__': 2,	'__gt__': 2,
     '__le__': 2,	'__lt__': 2,
 
     '__getattribute__': 2,	# only in new-style classes
+    '__get__': 3,		'__set__': 3,		'__delete__': 2,
     '__getattr__': 2,		'__setattr__': 3,	'__delattr__': 2,
     '__getitem__': 2,		'__setitem__': 3,	'__delitem__': 2,
     '__getslice__': 3,		'__setslice__': 4,	'__delslice__': 3,
@@ -352,8 +404,9 @@ SPECIAL_METHODS = {
     '__add__': 2,	'__radd__': 2,		'__iadd__': 2,    
     '__sub__': 2,	'__rsub__': 2,		'__isub__': 2,
     '__mul__': 2,	'__rmul__': 2,		'__imul__': 2,    
-    '__div__': 2,	'__rdiv__': 2,		'__idiv__': 2,    
-    '__pow__': 2,	'__rpow__': 2,		'__ipow__': 2,    # 2 or 3
+    '__div__': 2,	'__rdiv__': 2,		'__idiv__': 2,
+    #  __pow__: 2 or 3                           __ipow__: 2 or 3
+    '__pow__': 3,	'__rpow__': 2,		'__ipow__': 3,
     '__truediv__': 2,	'__rtruediv__': 2,	'__itruediv__': 2,	
     '__floordiv__': 2,	'__rfloordiv__': 2,	'__ifloordiv__': 2,	
     '__mod__': 2,	'__rmod__': 2,		'__imod__': 2,    
@@ -369,5 +422,12 @@ SPECIAL_METHODS = {
     # these are related to pickling 
     '__getstate__': 1,		'__setstate__': 2,
     '__copy__': 1,		'__deepcopy__': 2,
-    '__getinitargs__': 1,	
+    '__getinitargs__': 1,	'__getnewargs__': 1,
+    '__reduce__': 1,		'__reduce_ex__': 2,
     }
+
+if utils.pythonVersion() >= utils.PYTHON_2_5:
+    SPECIAL_METHODS['__enter__'] = 1
+    SPECIAL_METHODS['__exit__'] = 4
+
+NEW_STYLE_CLASS_METHODS = ['__getattribute__', '__set__', '__get__', '__delete__']
